`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2020/05/01 22:24:53
// Design Name: 
// Module Name: top
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

module top(
    input clk,
    input rst_n,
    input mode_ctl1,
    input mode_ctl2,
    input modeAMPM,
    input freq_sel0,
    input freq_sel1,
    input freq_sel2,
    output [7:0]segs,
    output [3:0]ssd_ctl
    );
    wire clk_1Hz, clk_100Hz, clk_10Hz, clk_10000Hz, clk_100kHz, clk_1MHz;
    wire [1:0]ssd_ctl_en;
    wire clk_out;
    wire [3:0]value_out1, value_out0;
    wire leap;
    wire [3:0]value_hour1, value_hour0, value_min1, value_min0, value_sec1, value_sec0;
    wire carry4, carry3, carry2, carry1, carry0;
    wire next_date, next_month, next_year;
    wire [3:0]limit0, limit1;
    wire [3:0]value_date0, value_date1;
    wire [3:0]value_month0, value_month1;
    wire [3:0]value_year0, value_year1, value_year2;
    wire carry_y0, carry_y1;
    wire [3:0]digit3, digit2, digit1, digit0;
    wire [3:0]ssd_in;


    freq_div U0(
        .clk(clk),
        .rst_n(rst_n),
        .clk_ctl(ssd_ctl_en),
        .clk_1Hz(clk_1Hz),
        .clk_10Hz(clk_10Hz),
        .clk_100Hz(clk_100Hz),
        .clk_10000Hz(clk_10000Hz),
        .clk_100kHz(clk_100kHz),
        .clk_1MHz(clk_1MHz)
    );

    freq_select U1(
        .select0(freq_sel0),
        .select1(freq_sel1),
        .select2(freq_sel2),
        .clk_1Hz(clk_1Hz),
        .clk_10Hz(clk_10Hz),
        .clk_100Hz(clk_100Hz),
        .clk_10000Hz(clk_10000Hz),
        .clk_100kHz(clk_100kHz),
        .clk_1MHz(clk_1MHz),
        .clk_100MHz(clk),
        .clk_out(clk_out)
    );

    BCD_up_cnt Usec0(
        .clk(clk_out),
        .rst_n(rst_n),
        .increase(1'b1),
        .limit(4'd9),
        .value(value_sec0),
        .carry(carry0)
    );

    BCD_up_cnt Usec1(
        .clk(clk_out),
        .rst_n(rst_n),
        .increase(carry0),
        .limit(4'd5),
        .value(value_sec1),
        .carry(carry1)
    );

    BCD_up_cnt Umin0(
        .clk(clk_out),
        .rst_n(rst_n),
        .increase(carry0 & carry1),
        .limit(4'd9),
        .value(value_min0),
        .carry(carry2)
    );

    BCD_up_cnt Umin1(
        .clk(clk_out),
        .rst_n(rst_n),
        .increase(carry0 & carry1 & carry2),
        .limit(4'd5),
        .value(value_min1),
        .carry(carry3)
    );

    BCD_up_cnt_hour0 Uhour0(
        .clk(clk_out),
        .rst_n(rst_n),
        .increase(carry0 & carry1 & carry2 & carry3),
        .limit(4'd9),
        .value_hour1(value_hour1),
        .value_hour0(value_hour0),
        .carry(carry4)
    );

    BCD_up_cnt_hour1 Uhour1(
        .clk(clk_out),
        .rst_n(rst_n),
        .increase(carry0 & carry1 & carry2 & carry3 & carry4),
        .limit(4'd2),
        .value_hour1(value_hour1),
        .value_hour0(value_hour0),
        .overflow(carry0 & carry1 & carry2 & carry3),
        .next_date(next_date)
    );

    counter_date Udate(
        .clk(clk_out),
        .rst_n(rst_n),
        .limit0(limit0),
        .limit1(limit1),
        .next_date(next_date),
        .next_month(next_month),
        .value0(value_date0),
        .value1(value_date1)
    );

    counter_month Umonth(
        .clk(clk_out),
        .rst_n(rst_n),
        .limit0(limit0),
        .limit1(limit1),
        .next_month(next_month),
        .next_year(next_year),
        .value0(value_month0),
        .value1(value_month1),
        .leap(leap)
    );

    leap_check Uleap(
        .clk(clk_out),
        .rst_n(rst_n),
        .next_year(next_year),
        .leap(leap)
    );

    counter_year Uyear0(
        .clk(clk_out),
        .rst_n(rst_n),
        .increase(next_year),
        .limit(4'd9),
        .value(value_year0),
        .carry(carry_y0)
    );

    counter_year Uyear1(
        .clk(clk_out),
        .rst_n(rst_n),
        .increase(carry_y0),
        .limit(4'd9),
        .value(value_year1),
        .carry(carry_y1)
    );

    counter_year Uyear2(
        .clk(clk_out),
        .rst_n(rst_n),
        .increase(carry_y1),
        .limit(4'd2),
        .value(value_year2)
    );

    mode U2(
        .mode(modeAMPM),
        .value_hour0(value_hour0),
        .value_hour1(value_hour1),
        .value_out0(value_out0),
        .value_out1(value_out1),
        .value_min0(value_min0),
        .value_min1(value_min1)
    );

    MUX U6(
        .digit3(digit3),
        .digit2(digit2),
        .digit1(digit1),
        .digit0(digit0),
        .mode_ctl1(mode_ctl1),
        .mode_ctl2(mode_ctl2),
        .digit3_month_day(value_month1),
        .digit2_month_day(value_month0),
        .digit1_month_day(value_date1),
        .digit0_month_day(value_date0),
        .digit2_year(value_year2),
        .digit1_year(value_year1),
        .digit0_year(value_year0),
        .digit3_hour_min(value_out1),
        .digit2_hour_min(value_out0),
        .digit1_hour_min(value_min1),
        .digit0_hour_min(value_min0),
        .digit1_sec(value_sec1),
        .digit0_sec(value_sec0)
    );

    scan_ctl U7(
        .ssd_ctl_en(ssd_ctl_en),
        .in3(digit3),
        .in2(digit2),
        .in1(digit1),
        .in0(digit0),
        .ssd_ctl(ssd_ctl),
        .ssd_in(ssd_in)
    );
    
    display U8(
        .in(ssd_in),
        .segs(segs)
    );

endmodule